﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using ArcGIS.Core.CIM;
using ArcGIS.Core.Data;
using ArcGIS.Core.Geometry;
using ArcGIS.Desktop.Catalog;
using ArcGIS.Desktop.Core;
using ArcGIS.Desktop.Editing;
using ArcGIS.Desktop.Extensions;
using ArcGIS.Desktop.Framework;
using ArcGIS.Desktop.Framework.Contracts;
using ArcGIS.Desktop.Framework.Dialogs;
using ArcGIS.Desktop.Framework.Threading.Tasks;
using ArcGIS.Desktop.Mapping;


namespace IntermediateEditing.Construction
{
	internal class ConstructFacilitiesTool2 : MapTool
	{
		public ConstructFacilitiesTool2()
		{
			IsSketchTool = true;
			UseSnapping = true;
			// Select the type of construction tool you wish to implement.  
			// Make sure that the tool is correctly registered with the correct component category type in the daml 
			SketchType = SketchGeometryType.Point;
			// SketchType = SketchGeometryType.Line;
			// SketchType = SketchGeometryType.Polygon;
			//Gets or sets whether the sketch is for creating a feature and should use the CurrentTemplate.
			UsesCurrentTemplate = true;
		}

		protected override Task OnToolDeactivateAsync(bool hasMapViewChanged)
		{
			if (!Module1.Current.SaveLastSubtypeChoiceToDefaults || CurrentTemplate == null)
			{
				return base.OnToolDeactivateAsync(hasMapViewChanged);
			}

			return QueuedTask.Run(() =>
			{
				var templateDef = this.CurrentTemplate.GetDefinition() as CIMFeatureTemplate;

				if (templateDef.DefaultValues == null)
					templateDef.DefaultValues = new Dictionary<string, object>();

				var choice = Module1.Current.SelectedSubtypeChoice;
				templateDef.DefaultValues["SUBTYPEFIELD"] = choice.SubtypefieldValue;
				templateDef.DefaultValues["FEATURECODE"] = choice.FeatureCodeValue;

				this.CurrentTemplate.SetDefinition(templateDef);
				Project.Current.SetDirty();//enable save
			});
		}
		/// <summary>
		/// Called when the sketch finishes. This is where we will create the sketch operation and then execute it.
		/// </summary>
		/// <param name="geometry">The geometry created by the sketch.</param>
		/// <returns>A Task returning a Boolean indicating if the sketch complete event was successfully handled.</returns>
		protected override Task<bool> OnSketchCompleteAsync(Geometry geometry)
		{

			if (CurrentTemplate == null || geometry == null)
				return Task.FromResult(false);

			return QueuedTask.Run(() =>
			{
				//get the template definition if we want access to the default values
				var templateDef = this.CurrentTemplate.GetDefinition() as CIMFeatureTemplate;
				string ownerVal = string.Empty;
				if (templateDef.DefaultValues?.ContainsKey("owner") ?? false)
					ownerVal = (string)templateDef.DefaultValues["owner"];

				//apply overrides
				if (Module1.Current.UseSubtypeChoiceOverride)
				{
					var choice = Module1.Current.SelectedSubtypeChoice;
					this.CurrentTemplate.Inspector["SUBTYPEFIELD"] = choice.SubtypefieldValue;
					this.CurrentTemplate.Inspector["FEATURECODE"] = choice.FeatureCodeValue;
				}

				// Create an edit operation
				var createOperation = new EditOperation();
				createOperation.Name = string.Format("Create {0}", CurrentTemplate.Layer.Name);
				createOperation.SelectNewFeatures = true;

				// Queue feature creation
				createOperation.Create(CurrentTemplate, geometry);

				// Execute the operation
				return createOperation.Execute();
			});
		}

		private Task Foo()
		{
			return QueuedTask.Run(() =>
			{
				CurrentTemplate.Layer.AutoGenerateTemplates(false);
				var templateDef = this.CurrentTemplate.GetDefinition() as CIMFeatureTemplate;

				if (templateDef.ReadDefaultToolDamlID() != this.ID)
					templateDef.SetDefaultToolDamlID(this.ID);

				var exclude_ids = new string[] { "esri_editing_SketchPointAtLineEndPointsTool" };
				//any of our "to be" excluded tools still shown on the palette
				var notexcluded = exclude_ids.Where(id => !templateDef.GetExcludedToolDamlIds().Contains(id)).ToArray();
				if (notexcluded.Count() > 0)
					templateDef.SetExcludedToolDamlIds(notexcluded);

				//specify tool options
				var all_tooloptions = templateDef.ToolOptions?.ToList() ?? new List<CIMEditingTemplateToolOptions>();
				var tooloptions = all_tooloptions.FirstOrDefault(to => to.ToolProgID == this.ID);
				if (tooloptions == null)
				{
					tooloptions = new CIMEditingTemplateToolOptions();
					tooloptions.Options = new Dictionary<string, object>();
					tooloptions.Options["SaveOverridesToDefault"] = false;
					tooloptions.ToolProgID = this.ID;
					all_tooloptions.Add(tooloptions);
				}
				templateDef.ToolOptions = all_tooloptions.ToArray();

				this.CurrentTemplate.SetDefinition(templateDef);
				return true;
			});
		}
	}
}
